!-----------------------------------------------------------------------------
! KINDANB: kinematic dynamo in a box
!                                           Akira Kageyama, kage@jamstec.go.jp
!                                              Earth Simulator Center, JAMSTEC
!-----------------------------------------------------------------------------
! avsdata.f90
!              2005.06.02: Minor revise. arguments of __write.
!              2005.06.01: Major revise.
!              2005.06.01: Copied from yycore project.
!-----------------------------------------------------------------------------

module avsdata
!*****************************************************************************
! MODULE AVSDATA                           Make and Save 3D Data in AVS Format
!*****************************************************************************
  ! - Automatic generation of AVS/Express's field data.
  ! - Makes use of my automatic avs-data-gerator "gavs.f90".
  !
  !                                                 2005.06.01  Akira Kageyama
  !---------------------------------------------------------------------------
  use grid
  use ut
  use field
  use gavs
  use namelist
  implicit none

  private
  public :: avsdata__initialize,  &
            avsdata__write

  integer :: Avs_nx  ! mesh size of avs data
  integer :: Avs_ny
  integer :: Avs_nz

  ! - 3D single precision real arrays.
  real(SP), dimension(:,:,:), allocatable :: Avs_vx  ! x-comp. of velocity
  real(SP), dimension(:,:,:), allocatable :: Avs_vy  ! y-comp.
  real(SP), dimension(:,:,:), allocatable :: Avs_vz  ! z-comp.
  real(SP), dimension(:,:,:), allocatable :: Avs_bx  ! x-comp. of mag. field
  real(SP), dimension(:,:,:), allocatable :: Avs_by  !
  real(SP), dimension(:,:,:), allocatable :: Avs_bz  !
  real(SP), dimension(:,:,:), allocatable :: Avs_jx  ! x-comp. of current.
  real(SP), dimension(:,:,:), allocatable :: Avs_jy  !
  real(SP), dimension(:,:,:), allocatable :: Avs_jz  !

  !--- Coordinates information for AVS field data ---
  real(SP), dimension(:,:,:), allocatable :: Avs_coord_x
  real(SP), dimension(:,:,:), allocatable :: Avs_coord_y
  real(SP), dimension(:,:,:), allocatable :: Avs_coord_z

contains


!===============
!    Private
!===============


!_______________________________________________________________private__
!                                                                        !
  subroutine make_single_precision_field(mag,vel)                        !
    type(field__vector3d_), intent(in) :: mag, vel                       !
!________________________________________________________________________!
!
    type(field__vector3d_) :: cur   ! electrical current density

    cur = .curl.mag

    Avs_vx = real(vel%x,SP)
    Avs_vy = real(vel%y,SP)
    Avs_vz = real(vel%z,SP)
    Avs_bx = real(mag%x,SP)
    Avs_by = real(mag%y,SP)
    Avs_bz = real(mag%z,SP)
    Avs_jx = real(cur%x,SP)
    Avs_jy = real(cur%y,SP)
    Avs_jz = real(cur%z,SP)

  end subroutine make_single_precision_field


!_______________________________________________________________private__
!                                                                        !
  subroutine output(nth_call)                                            !
    character(len=3), intent(in) :: nth_call                             !
!________________________________________________________________________!
!
    logical :: first = .true.              ! Automatic save attribute.

    if (first) then                        ! Save coordinate data
       call gavs__coord(FILE_AVS_DATA,                          &
                        trim(namelist__string('Avs_tag')),      &
                        Avs_nx, Avs_ny, Avs_nz,                 &
                        Avs_coord_x,                            &
                        Avs_coord_y,                            &
                        Avs_coord_z)
       first = .false.
    end if

    call gavs__fld(FILE_AVS_DATA,                                    &
                   trim(namelist__string('Avs_tag')),                &
                   trim(namelist__string('Avs_tag'))//'.'//nth_call, &
                   Avs_nx, Avs_ny, Avs_nz,                           &
                   Avs_vx, 'vx',                                     &
                   Avs_vy, 'vy',                                     &
                   Avs_vz, 'vz',                                     &
                   Avs_bx, 'bx',                                     &
                   Avs_by, 'by',                                     &
                   Avs_bz, 'bz',                                     &
                   Avs_jz, 'jx',                                     &
                   Avs_jy, 'jy',                                     &
                   Avs_jz, 'jz')

  end subroutine output


!==============
!    Public
!==============


!________________________________________________________________public__
!                                                                        !
  subroutine avsdata__initialize                                         !
!________________________________________________________________________!
!
    integer  :: i, j, k

    Avs_nx = NX    ! You may want to reduce the output data size here.
    Avs_ny = NY
    Avs_nz = NZ

    call iMalloc(Avs_nx,Avs_ny,Avs_nz)

    do k = 1 , Avs_nz
       do j = 1 , Avs_ny
          do i = 1 , Avs_nx
             Avs_coord_x(i,j,k) = real(grid__pos%x(i),SP)
             Avs_coord_y(i,j,k) = real(grid__pos%y(j),SP)
             Avs_coord_z(i,j,k) = real(grid__pos%z(k),SP)
          end do
       end do
    end do

  contains

  !- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -!
    subroutine iMalloc(nx,ny,nz)
      integer, intent(in) :: nx, ny, nz
  !- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -!

      allocate(Avs_vx(nx,ny,nz),         &
               Avs_vy(nx,ny,nz),         &
               Avs_vz(nx,ny,nz),         &
               Avs_bx(nx,ny,nz),         &
               Avs_by(nx,ny,nz),         &
               Avs_bz(nx,ny,nz),         &
               Avs_jx(nx,ny,nz),         &
               Avs_jy(nx,ny,nz),         &
               Avs_jz(nx,nx,ny))

      allocate(Avs_coord_x(nx,ny,nz),    &
               Avs_coord_y(nx,ny,nz),    &
               Avs_coord_z(nx,ny,nz))

    end subroutine iMalloc

  end subroutine avsdata__initialize


!________________________________________________________________public__
!                                                                        !
  subroutine avsdata__write(vel,mag,nloop,time)                          !
    type(field__vector3d_), intent(in) :: vel, mag                       !
    integer,                intent(in) :: nloop                          !
    real(DP),               intent(in) :: time                           !
!________________________________________________________________________!
!
    integer :: counter = 0               ! it has save attribute.
    character(len=3) :: nth_call

    if ( namelist__integer('Avs_nskip') <= 0 ) return    
                                      ! Set zero or negative integer
                                      ! when you don't want to
                                      ! save any AVS data.

    if (mod(nloop,namelist__integer('Avs_nskip')) /= 0) return

    call make_single_precision_field(mag,vel)

    call ut__i2c3(counter, nth_call)
    call output(nth_call)

    call ut__message('#avs data saved: '//nth_call, nloop, time)

    counter = counter + 1

  end subroutine avsdata__write

end module avsdata
