!-----------------------------------------------------------------------------
! KINDANB: kinematic dynamo in a box
!-----------------------------------------------------------------------------
! field.f90
!              2005.09.25: Use grid, no global.
!              2005.06.02: Forked from global.f90.
!              2005.06.02: Created by Akira Kageyama.
!-----------------------------------------------------------------------------

module field
!*****************************************************************************
! MODULE FIELD                                  Field Operators and Operations
!*****************************************************************************
  use constants
  use grid
  implicit none

  public
  private :: assignment_scalar_to_vector3d
  private :: operator_add,                      &
             operator_cross,                    &
             operator_curl,                     &
             operator_div,                      &
             operator_energyintegral,           &
             operator_laplacian,                &
             operator_normalize,                &
             operator_vector3d_times_scalar,    &
             operator_scalar_times_vector3d

  interface field__boundary_condition
     module procedure boundary_condition_scalar, &
                      boundary_condition_vector
  end interface

  !--- << Types >> ---!

  type field__vector3d_
     real(DP), dimension(NX,NY,NZ) :: x
     real(DP), dimension(NX,NY,NZ) :: y
     real(DP), dimension(NX,NY,NZ) :: z
  end type field__vector3d_

  !--- << Operators >> ---!

  interface operator(.curl.)
     module procedure operator_curl
  end interface

  interface operator(.div.)
     module procedure operator_div
  end interface

  interface operator(.energyintegral.)
     module procedure operator_energyintegral
  end interface

  interface operator(.laplacian.)
     module procedure operator_laplacian
  end interface

  interface operator(.normalize.)
     module procedure operator_normalize
  end interface

  interface operator(.x.)
     module procedure operator_cross
  end interface

  interface operator(+)
     module procedure operator_add
  end interface

  interface operator(*)
     module procedure operator_scalar_times_vector3d
     module procedure operator_vector3d_times_scalar
  end interface

  interface assignment(=)
     module procedure assignment_scalar_to_vector3d
  end interface


contains


!===============
!    Private
!===============

!_______________________________________________________________private__
!                                                                        !
  subroutine assignment_scalar_to_vector3d(vector3d,scalar)              !
    type(field__vector3d_), intent(out) :: vector3d                      !
    real(DP),         intent(in)  :: scalar                              !
!________________________________________________________________________!
!
    vector3d%x(:,:,:) = scalar
    vector3d%y(:,:,:) = scalar
    vector3d%z(:,:,:) = scalar

  end subroutine assignment_scalar_to_vector3d


!_______________________________________________________________private__
!                                                                        !
  function operator_add(a,b)                                             !
    type(field__vector3d_), intent(in) :: a, b                           !
    type(field__vector3d_)             :: operator_add                   !
!________________________________________________________________________!
!
    operator_add%x = a%x + b%x
    operator_add%y = a%y + b%y
    operator_add%z = a%z + b%z

  end function operator_add


!_______________________________________________________________private__
!                                                                        !
  function operator_curl(a)                                              !
    type(field__vector3d_), intent(in) :: a                              !
    type(field__vector3d_)             :: operator_curl                  !
!________________________________________________________________________!
!
    integer  :: i, j, k
    real(DP) :: dx1, dy1, dz1

    dx1 = grid__d1%x
    dy1 = grid__d1%y
    dz1 = grid__d1%z

    do k = 2 , NZ-1
       do j = 2 , NY-1
          do i = 2 , NX-1
             operator_curl%x(i,j,k) = dy1*(a%z(i,j+1,k)-a%z(i,j-1,k)) &
                                    - dz1*(a%y(i,j,k+1)-a%y(i,j,k-1))
             operator_curl%y(i,j,k) = dz1*(a%x(i,j,k+1)-a%x(i,j,k-1)) &
                                    - dx1*(a%z(i+1,j,k)-a%z(i-1,j,k))
             operator_curl%z(i,j,k) = dx1*(a%y(i+1,j,k)-a%y(i-1,j,k)) &
                                    - dy1*(a%x(i,j+1,k)-a%x(i,j-1,k))
          end do
       end do
    end do

    call boundary_condition_vector(operator_curl)

  end function operator_curl


!_______________________________________________________________private__
!                                                                        !
  function operator_div(a)                                               !
    type(field__vector3d_), intent(in)  :: a                             !
    real(DP), dimension(NX,NY,NZ) :: operator_div                        !
!________________________________________________________________________!
!
    integer  :: i, j, k
    real(DP) :: dx1, dy1, dz1

    dx1 = grid__d1%x
    dy1 = grid__d1%y
    dz1 = grid__d1%z

    do k = 2 , NZ-1
       do j = 2 , NY-1
          do i = 2 , NX-1
             operator_div(i,j,k) = dx1*(a%x(i+1,j,k)-a%x(i-1,j,k)) &
                                 + dy1*(a%y(i,j+1,k)-a%y(i,j-1,k)) &
                                 + dz1*(a%z(i,j,k+1)-a%z(i,j,k-1))
          end do
       end do
    end do

    call boundary_condition_scalar(operator_div)

  end function operator_div


!_______________________________________________________________private__
!                                                                        !
  function operator_cross(a,b)                                           !
    type(field__vector3d_), intent(in) :: a, b                           !
    type(field__vector3d_)             :: operator_cross                 !
!________________________________________________________________________!
!
    operator_cross%x = (a%y)*(b%z) - (a%z)*(b%y)
    operator_cross%y = (a%z)*(b%x) - (a%x)*(b%z)
    operator_cross%z = (a%x)*(b%y) - (a%y)*(b%x)

  end function operator_cross


!_______________________________________________________________private__
!                                                                        !
  function operator_laplacian(a)                                         !
    type(field__vector3d_), intent(in) :: a                              !
    type(field__vector3d_)             :: operator_laplacian             !
!________________________________________________________________________!
!
    integer  :: i, j, k
    real(DP) :: dx2, dy2, dz2

    dx2 = grid__d2%x
    dy2 = grid__d2%y
    dz2 = grid__d2%z

    do k = 2 , NZ-1
       do j = 2 , NY-1
          do i = 2 , NX-1
             operator_laplacian%x(i,j,k)                                &
                  = dx2*(a%x(i+1,j,k)-2*a%x(i,j,k)+a%x(i-1,j,k))        &
                  + dy2*(a%x(i,j+1,k)-2*a%x(i,j,k)+a%x(i,j-1,k))        &
                  + dz2*(a%x(i,j,k+1)-2*a%x(i,j,k)+a%x(i,j,k-1))
             operator_laplacian%y(i,j,k)                                &
                  = dx2*(a%y(i+1,j,k)-2*a%y(i,j,k)+a%y(i-1,j,k))        &
                  + dy2*(a%y(i,j+1,k)-2*a%y(i,j,k)+a%y(i,j-1,k))        &
                  + dz2*(a%y(i,j,k+1)-2*a%y(i,j,k)+a%y(i,j,k-1))
             operator_laplacian%z(i,j,k)                                &
                  = dx2*(a%z(i+1,j,k)-2*a%z(i,j,k)+a%z(i-1,j,k))        &
                  + dy2*(a%z(i,j+1,k)-2*a%z(i,j,k)+a%z(i,j-1,k))        &
                  + dz2*(a%z(i,j,k+1)-2*a%z(i,j,k)+a%z(i,j,k-1))
          end do
       end do
    end do

    call boundary_condition_vector(operator_laplacian)

  end function operator_laplacian


!_______________________________________________________________private__
!                                                                        !
  function operator_energyintegral(a)                                    !
    type(field__vector3d_), intent(in) :: a                              !
    real(DP)                     :: operator_energyintegral              !
!________________________________________________________________________!
!
    real(DP) :: dvol

    dvol = (grid__delta%x)*(grid__delta%y)*(grid__delta%z)

    operator_energyintegral                             &
         = 0.5_DP * sum(  a%x(2:NX-1,2:NY-1,2:NZ-1)**2  &
                         +a%y(2:NX-1,2:NY-1,2:NZ-1)**2  &
                         +a%z(2:NX-1,2:NY-1,2:NZ-1)**2  &
                        ) * dvol

  end function operator_energyintegral


!_______________________________________________________________private__
!                                                                        !
  function operator_normalize(a)                                         !
    type(field__vector3d_), intent(in) :: a                              !
    type(field__vector3d_)             :: operator_normalize             !
!________________________________________________________________________!
!
    real(DP) :: factor

    factor = 1.0_DP / (.energyintegral.a)

    operator_normalize = factor*a

  end function operator_normalize

!_______________________________________________________________private__
!                                                                        !
  function operator_scalar_times_vector3d(scalar,vec)                    !
    real(DP),         intent(in) :: scalar                               !
    type(field__vector3d_), intent(in) :: vec                            !
    type(field__vector3d_)             :: operator_scalar_times_vector3d !
!________________________________________________________________________!
!
    operator_scalar_times_vector3d%x = scalar*(vec%x)
    operator_scalar_times_vector3d%y = scalar*(vec%y)
    operator_scalar_times_vector3d%z = scalar*(vec%z)

  end function operator_scalar_times_vector3d


!_______________________________________________________________private__
!                                                                        !
  function operator_vector3d_times_scalar(vec,scalar)                    !
    type(field__vector3d_), intent(in) :: vec                            !
    real(DP),               intent(in) :: scalar                         !
    type(field__vector3d_)             :: operator_vector3d_times_scalar !
!________________________________________________________________________!
!
    operator_vector3d_times_scalar%x = scalar*(vec%x)
    operator_vector3d_times_scalar%y = scalar*(vec%y)
    operator_vector3d_times_scalar%z = scalar*(vec%z)

  end function operator_vector3d_times_scalar


!_______________________________________________________________private__
!                                                                        !
  subroutine boundary_condition_scalar(scalar)                           !
    real(DP), dimension(NX,NY,NZ), intent(inout) :: scalar               !
!________________________________________________________________________!
!
    scalar( 1,:,:) = scalar(NX-1,:,:)
    scalar(NX,:,:) = scalar(   2,:,:)

    scalar(:, 1,:) = scalar(:,NY-1,:)
    scalar(:,NY,:) = scalar(:,   2,:)

    scalar(:,:, 1) = scalar(:,:,NZ-1)
    scalar(:,:,NZ) = scalar(:,:,   2)

  end subroutine boundary_condition_scalar


!_______________________________________________________________private__
!                                                                        !
  subroutine boundary_condition_vector(vec)                              !
    type(field__vector3d_), intent(inout) :: vec                         !
!________________________________________________________________________!
!
    vec%x( 1,:,:) = vec%x(NX-1,:,:)    !-- yz-plane --!
    vec%y( 1,:,:) = vec%y(NX-1,:,:)
    vec%z( 1,:,:) = vec%z(NX-1,:,:)
    vec%x(NX,:,:) = vec%x(   2,:,:)
    vec%y(NX,:,:) = vec%y(   2,:,:)
    vec%z(NX,:,:) = vec%z(   2,:,:)

    vec%x(:, 1,:) = vec%x(:,NY-1,:)    !-- zx-plane --!
    vec%y(:, 1,:) = vec%y(:,NY-1,:)
    vec%z(:, 1,:) = vec%z(:,NY-1,:)
    vec%x(:,NY,:) = vec%x(:,   2,:)
    vec%y(:,NY,:) = vec%y(:,   2,:)
    vec%z(:,NY,:) = vec%z(:,   2,:)

    vec%x(:,:, 1) = vec%x(:,:,NZ-1)    !-- xy-plane --!
    vec%y(:,:, 1) = vec%y(:,:,NZ-1)
    vec%z(:,:, 1) = vec%z(:,:,NZ-1)
    vec%x(:,:,NZ) = vec%x(:,:,   2)
    vec%y(:,:,NZ) = vec%y(:,:,   2)
    vec%z(:,:,NZ) = vec%z(:,:,   2)

  end subroutine boundary_condition_vector

end module field

