!-----------------------------------------------------------------------------
! KINDANB: kinematic dynamo in a box
!                                           Akira Kageyama, kage@jamstec.go.jp
!                                              Earth Simulator Center, JAMSTEC
!-----------------------------------------------------------------------------
! gavs.f90
!              2003.05.21: Created by Akira Kageyama
!-----------------------------------------------------------------------------

module gavs
!*****************************************************************************
! MODULE GAVS                              Generator of AVS-Field Data (*.fld)
!*****************************************************************************
  ! 2003.05.21  by Akira Kageyama (ESC)
  !   - Removed 'target_dir' funcion, for ES use.
  !   - Separated the gavs__fld routine into two part:
  !     for coordinate data and field data.
  !
  ! 2003.02.06  by Akira Kageyama (ESC)
  !   - Added ids4avs; removed IDS_TEMP (no more problem for ids=77).
  !   - Renamed makeAVs --> gavs.
  !
  ! 2003.02.04  by Akira Kageyama (ESC)
  !   makeAvs.f90
  !     I converted old f77maf-lib.f into fortran90 version.
  !     In the old version, you had to call a companion shell script.
  !     In this new version, everything is done as a f90 program. 
  !
  ! 1995.02.04  by Kageyama, Akira (NIFS)
  !   f77maf
  !     To make an AVS-field data file from a fortran code.
  !     
  !---------------------------------------------------------------------------
  ! Usage:
  !    call gavs__coord(IDS_FOR_AVS,                      &
  !                     tag_coord,                        &
  !                     nr, nt, np,                       &
  !                     coord%x, coord%y, coord%z)
  !
  !    call gavs__fld(IDS_FOR_AVS,                        &
  !                   tag_coord,                          &
  !                   tag_fld,                            &
  !                   nr, nt, np,                         &
  !                   data_avs_vel%x, 'vx',               &
  !                   data_avs_vel%y, 'vy',               &
  !                   data_avs_vel%z, 'vz',               &
  !                   data_avs_mag%x, 'bx',               &
  !                   data_avs_mag%y, 'by',               &
  !                   data_avs_mag%z, 'bz')
  !---------------------------------------------------------------------------
  use constants, only: SP
  implicit none
  private
  public :: gavs__coord,  &
            gavs__fld
  
  interface gavs__fld
     module procedure avs01f
     module procedure avs02f
     module procedure avs03f
     module procedure avs04f
     module procedure avs05f
     module procedure avs06f
     module procedure avs07f
     module procedure avs08f
     module procedure avs09f
     module procedure avs10f
  end interface


contains

!===============
!    Private    
!===============
    
  subroutine avs01f(ids4avs,       &
                   tag_coord,      &
                   tag_fld,        &
                   n1, n2, n3,     &
                   vec1, lab1)
    integer, intent(in) :: ids4avs
    character(len=*), intent(in) :: tag_coord, tag_fld
    integer, intent(in) :: n1, n2, n3
    real(SP), dimension(n1,n2,n3), intent(in) :: vec1
    character(len=*), intent(in) :: lab1
!------------------------------------------------------------
! SUBROUTINE AVS01F
!------------------------------------------------------------
    integer :: nvec = 1

    call write_3d_data(ids4avs, n1, n2, n3, tag_fld//'.'//lab1, vec1)

    open(ids4avs,file=tag_fld//'.fld',form='formatted')
      write(ids4avs,'(a)')    '# AVS field data (by gavs__avs01f)'
      write(ids4avs,'(a)')    '#'
      write(ids4avs,'(a,i5)') 'ndim = ',3
      write(ids4avs,'(a,i5)') 'dim1 = ',n1
      write(ids4avs,'(a,i5)') 'dim2 = ',n2
      write(ids4avs,'(a,i5)') 'dim3 = ',n3
      write(ids4avs,'(a,i5)') 'nspace = ',3
      write(ids4avs,'(a,i5)') 'veclen = ',nvec
      write(ids4avs,'(a)')    'data = float'
      write(ids4avs,'(a)')    'field = irregular'
      write(ids4avs,'(a)')    'label = '//lab1
      write(ids4avs,'(a)')    '#'
      write(ids4avs,'(a)')    'coord 1 file='//tag_coord//'.x filetype=unformatted'
      write(ids4avs,'(a)')    'coord 2 file='//tag_coord//'.y filetype=unformatted'
      write(ids4avs,'(a)')    'coord 3 file='//tag_coord//'.z filetype=unformatted'
      write(ids4avs,'(a)')    '#'
      write(ids4avs,'(a)')    &
                     'variable 1 file='//tag_fld//'.'//lab1//' filetype=unformatted'
    close(ids4avs)
  end subroutine avs01f
    

  subroutine avs02f(ids4avs,         &
                   tag_coord,        &
                   tag_fld,          &
                   n1, n2, n3,       &
                   vec1, lab1,       &
                   vec2, lab2        &
                   ) 
    integer, intent(in) :: ids4avs
    character(len=*), intent(in) :: tag_coord, tag_fld
    integer, intent(in) :: n1, n2, n3
    real(SP), dimension(n1,n2,n3), intent(in) :: vec1
    real(SP), dimension(n1,n2,n3), intent(in) :: vec2
    character(len=*), intent(in) :: lab1
    character(len=*), intent(in) :: lab2
!------------------------------------------------------------
! SUBROUTINE AVS02F
!------------------------------------------------------------
    integer :: nvec = 2

    call write_3d_data(ids4avs, n1, n2, n3, tag_fld//'.'//lab1, vec1)
    call write_3d_data(ids4avs, n1, n2, n3, tag_fld//'.'//lab2, vec2)

    open(ids4avs,file=tag_fld//'.fld',form='formatted')
      write(ids4avs,'(a)')    '# AVS field data (by gavs__avs02f)'
      write(ids4avs,'(a)')    '#'
      write(ids4avs,'(a,i5)') 'ndim = ',3
      write(ids4avs,'(a,i5)') 'dim1 = ',n1
      write(ids4avs,'(a,i5)') 'dim2 = ',n2
      write(ids4avs,'(a,i5)') 'dim3 = ',n3
      write(ids4avs,'(a,i5)') 'nspace = ',3
      write(ids4avs,'(a,i5)') 'veclen = ',nvec
      write(ids4avs,'(a)')    'data = float'
      write(ids4avs,'(a)')    'field = irregular'
      write(ids4avs,'(a)')    'label = '//lab1
      write(ids4avs,'(a)')    'label = '//lab2
      write(ids4avs,'(a)')    '#'
      write(ids4avs,'(a)')    'coord 1 file='//tag_coord//'.x filetype=unformatted'
      write(ids4avs,'(a)')    'coord 2 file='//tag_coord//'.y filetype=unformatted'
      write(ids4avs,'(a)')    'coord 3 file='//tag_coord//'.z filetype=unformatted'
      write(ids4avs,'(a)')    '#'
      write(ids4avs,'(a)')    &
                     'variable 1 file='//tag_fld//'.'//lab1//' filetype=unformatted'
      write(ids4avs,'(a)')    &
                     'variable 2 file='//tag_fld//'.'//lab2//' filetype=unformatted'
    close(ids4avs)
  end subroutine avs02f
    

  subroutine avs03f(ids4avs,       &
                   tag_coord,      &
                   tag_fld,        &
                   n1, n2, n3,     &
                   vec1, lab1,     &
                   vec2, lab2,     &
                   vec3, lab3      &
                   )
    integer, intent(in) :: ids4avs
    character(len=*), intent(in) :: tag_coord, tag_fld
    integer, intent(in) :: n1, n2, n3
    real(SP), dimension(n1,n2,n3), intent(in) :: vec1
    real(SP), dimension(n1,n2,n3), intent(in) :: vec2
    real(SP), dimension(n1,n2,n3), intent(in) :: vec3
    character(len=*), intent(in) :: lab1
    character(len=*), intent(in) :: lab2
    character(len=*), intent(in) :: lab3
!------------------------------------------------------------
! SUBROUTINE AVS03F
!------------------------------------------------------------
    integer :: nvec = 3

    call write_3d_data(ids4avs, n1, n2, n3, tag_fld//'.'//lab1, vec1)
    call write_3d_data(ids4avs, n1, n2, n3, tag_fld//'.'//lab2, vec2)
    call write_3d_data(ids4avs, n1, n2, n3, tag_fld//'.'//lab3, vec3)

    open(ids4avs,file=tag_fld//'.fld',form='formatted')
      write(ids4avs,'(a)')    '# AVS field data (by gavs__avs03f)'
      write(ids4avs,'(a)')    '#'
      write(ids4avs,'(a,i5)') 'ndim = ',3
      write(ids4avs,'(a,i5)') 'dim1 = ',n1
      write(ids4avs,'(a,i5)') 'dim2 = ',n2
      write(ids4avs,'(a,i5)') 'dim3 = ',n3
      write(ids4avs,'(a,i5)') 'nspace = ',3
      write(ids4avs,'(a,i5)') 'veclen = ',nvec
      write(ids4avs,'(a)')    'data = float'
      write(ids4avs,'(a)')    'field = irregular'
      write(ids4avs,'(a)')    'label = '//lab1
      write(ids4avs,'(a)')    'label = '//lab2
      write(ids4avs,'(a)')    'label = '//lab3
      write(ids4avs,'(a)')    '#'
      write(ids4avs,'(a)')    'coord 1 file='//tag_coord//'.x filetype=unformatted'
      write(ids4avs,'(a)')    'coord 2 file='//tag_coord//'.y filetype=unformatted'
      write(ids4avs,'(a)')    'coord 3 file='//tag_coord//'.z filetype=unformatted'
      write(ids4avs,'(a)')    '#'
      write(ids4avs,'(a)')    &
                     'variable 1 file='//tag_fld//'.'//lab1//' filetype=unformatted'
      write(ids4avs,'(a)')    &
                     'variable 2 file='//tag_fld//'.'//lab2//' filetype=unformatted'
      write(ids4avs,'(a)')    &
                     'variable 3 file='//tag_fld//'.'//lab3//' filetype=unformatted'
    close(ids4avs)
  end subroutine avs03f
    

  subroutine avs04f(ids4avs,       &
                   tag_coord,      &
                   tag_fld,        &
                   n1, n2, n3,     &
                   vec1, lab1,     &
                   vec2, lab2,     &
                   vec3, lab3,     &
                   vec4, lab4      &
                   )
    integer, intent(in) :: ids4avs
    character(len=*), intent(in) :: tag_coord, tag_fld
    integer, intent(in) :: n1, n2, n3
    real(SP), dimension(n1,n2,n3), intent(in) :: vec1
    real(SP), dimension(n1,n2,n3), intent(in) :: vec2
    real(SP), dimension(n1,n2,n3), intent(in) :: vec3
    real(SP), dimension(n1,n2,n3), intent(in) :: vec4
    character(len=*), intent(in) :: lab1
    character(len=*), intent(in) :: lab2
    character(len=*), intent(in) :: lab3
    character(len=*), intent(in) :: lab4
!------------------------------------------------------------
! SUBROUTINE AVS04F
!------------------------------------------------------------
    integer :: nvec = 4

    call write_3d_data(ids4avs, n1, n2, n3, tag_fld//'.'//lab1, vec1)
    call write_3d_data(ids4avs, n1, n2, n3, tag_fld//'.'//lab2, vec2)
    call write_3d_data(ids4avs, n1, n2, n3, tag_fld//'.'//lab3, vec3)
    call write_3d_data(ids4avs, n1, n2, n3, tag_fld//'.'//lab4, vec4)

    open(ids4avs,file=tag_fld//'.fld',form='formatted')
      write(ids4avs,'(a)')    '# AVS field data (by gavs__avs04f)'
      write(ids4avs,'(a)')    '#'
      write(ids4avs,'(a,i5)') 'ndim = ',3
      write(ids4avs,'(a,i5)') 'dim1 = ',n1
      write(ids4avs,'(a,i5)') 'dim2 = ',n2
      write(ids4avs,'(a,i5)') 'dim3 = ',n3
      write(ids4avs,'(a,i5)') 'nspace = ',3
      write(ids4avs,'(a,i5)') 'veclen = ',nvec
      write(ids4avs,'(a)')    'data = float'
      write(ids4avs,'(a)')    'field = irregular'
      write(ids4avs,'(a)')    'label = '//lab1
      write(ids4avs,'(a)')    'label = '//lab2
      write(ids4avs,'(a)')    'label = '//lab3
      write(ids4avs,'(a)')    'label = '//lab4
      write(ids4avs,'(a)')    '#'
      write(ids4avs,'(a)')    'coord 1 file='//tag_coord//'.x filetype=unformatted'
      write(ids4avs,'(a)')    'coord 2 file='//tag_coord//'.y filetype=unformatted'
      write(ids4avs,'(a)')    'coord 3 file='//tag_coord//'.z filetype=unformatted'
      write(ids4avs,'(a)')    '#'
      write(ids4avs,'(a)')    &
                     'variable 1 file='//tag_fld//'.'//lab1//' filetype=unformatted'
      write(ids4avs,'(a)')    &
                     'variable 2 file='//tag_fld//'.'//lab2//' filetype=unformatted'
      write(ids4avs,'(a)')    &
                     'variable 3 file='//tag_fld//'.'//lab3//' filetype=unformatted'
      write(ids4avs,'(a)')    &
                     'variable 4 file='//tag_fld//'.'//lab4//' filetype=unformatted'
    close(ids4avs)
  end subroutine avs04f
    

  subroutine avs05f(ids4avs,        &
                   tag_coord,       &
                   tag_fld,         &
                   n1, n2, n3,      &
                   vec1, lab1,      &
                   vec2, lab2,      &
                   vec3, lab3,      &
                   vec4, lab4,      &
                   vec5, lab5       &
                   )
    integer, intent(in) :: ids4avs
    character(len=*), intent(in) :: tag_coord, tag_fld
    integer, intent(in) :: n1, n2, n3
    real(SP), dimension(n1,n2,n3), intent(in) :: vec1
    real(SP), dimension(n1,n2,n3), intent(in) :: vec2
    real(SP), dimension(n1,n2,n3), intent(in) :: vec3
    real(SP), dimension(n1,n2,n3), intent(in) :: vec4
    real(SP), dimension(n1,n2,n3), intent(in) :: vec5
    character(len=*), intent(in) :: lab1
    character(len=*), intent(in) :: lab2
    character(len=*), intent(in) :: lab3
    character(len=*), intent(in) :: lab4
    character(len=*), intent(in) :: lab5
!------------------------------------------------------------
! SUBROUTINE AVS05F
!------------------------------------------------------------
    integer :: nvec = 5

    call write_3d_data(ids4avs, n1, n2, n3, tag_fld//'.'//lab1, vec1)
    call write_3d_data(ids4avs, n1, n2, n3, tag_fld//'.'//lab2, vec2)
    call write_3d_data(ids4avs, n1, n2, n3, tag_fld//'.'//lab3, vec3)
    call write_3d_data(ids4avs, n1, n2, n3, tag_fld//'.'//lab4, vec4)
    call write_3d_data(ids4avs, n1, n2, n3, tag_fld//'.'//lab5, vec5)

    open(ids4avs,file=tag_fld//'.fld',form='formatted')
      write(ids4avs,'(a)')    '# AVS field data (by gavs__avs05f)'
      write(ids4avs,'(a)')    '#'
      write(ids4avs,'(a,i5)') 'ndim = ',3
      write(ids4avs,'(a,i5)') 'dim1 = ',n1
      write(ids4avs,'(a,i5)') 'dim2 = ',n2
      write(ids4avs,'(a,i5)') 'dim3 = ',n3
      write(ids4avs,'(a,i5)') 'nspace = ',3
      write(ids4avs,'(a,i5)') 'veclen = ',nvec
      write(ids4avs,'(a)')    'data = float'
      write(ids4avs,'(a)')    'field = irregular'
      write(ids4avs,'(a)')    'label = '//lab1
      write(ids4avs,'(a)')    'label = '//lab2
      write(ids4avs,'(a)')    'label = '//lab3
      write(ids4avs,'(a)')    'label = '//lab4
      write(ids4avs,'(a)')    'label = '//lab5
      write(ids4avs,'(a)')    '#'
      write(ids4avs,'(a)')    'coord 1 file='//tag_coord//'.x filetype=unformatted'
      write(ids4avs,'(a)')    'coord 2 file='//tag_coord//'.y filetype=unformatted'
      write(ids4avs,'(a)')    'coord 3 file='//tag_coord//'.z filetype=unformatted'
      write(ids4avs,'(a)')    '#'
      write(ids4avs,'(a)')    &
                     'variable 1 file='//tag_fld//'.'//lab1//' filetype=unformatted'
      write(ids4avs,'(a)')    &
                     'variable 2 file='//tag_fld//'.'//lab2//' filetype=unformatted'
      write(ids4avs,'(a)')    &
                     'variable 3 file='//tag_fld//'.'//lab3//' filetype=unformatted'
      write(ids4avs,'(a)')    &
                     'variable 4 file='//tag_fld//'.'//lab4//' filetype=unformatted'
      write(ids4avs,'(a)')    &
                     'variable 5 file='//tag_fld//'.'//lab5//' filetype=unformatted'
    close(ids4avs)
  end subroutine avs05f
    

  subroutine avs06f(ids4avs,        &
                   tag_coord,       &
                   tag_fld,         &
                   n1, n2, n3,      &
                   vec1, lab1,      &
                   vec2, lab2,      &
                   vec3, lab3,      &
                   vec4, lab4,      &
                   vec5, lab5,      &
                   vec6, lab6       &
                   )
    integer, intent(in) :: ids4avs
    character(len=*), intent(in) :: tag_coord, tag_fld
    integer, intent(in) :: n1, n2, n3
    real(SP), dimension(n1,n2,n3), intent(in) :: vec1
    real(SP), dimension(n1,n2,n3), intent(in) :: vec2
    real(SP), dimension(n1,n2,n3), intent(in) :: vec3
    real(SP), dimension(n1,n2,n3), intent(in) :: vec4
    real(SP), dimension(n1,n2,n3), intent(in) :: vec5
    real(SP), dimension(n1,n2,n3), intent(in) :: vec6
    character(len=*), intent(in) :: lab1
    character(len=*), intent(in) :: lab2
    character(len=*), intent(in) :: lab3
    character(len=*), intent(in) :: lab4
    character(len=*), intent(in) :: lab5
    character(len=*), intent(in) :: lab6
!------------------------------------------------------------
! SUBROUTINE AVS06F
!------------------------------------------------------------
    integer :: nvec = 6

    call write_3d_data(ids4avs, n1, n2, n3, tag_fld//'.'//lab1, vec1)
    call write_3d_data(ids4avs, n1, n2, n3, tag_fld//'.'//lab2, vec2)
    call write_3d_data(ids4avs, n1, n2, n3, tag_fld//'.'//lab3, vec3)
    call write_3d_data(ids4avs, n1, n2, n3, tag_fld//'.'//lab4, vec4)
    call write_3d_data(ids4avs, n1, n2, n3, tag_fld//'.'//lab5, vec5)
    call write_3d_data(ids4avs, n1, n2, n3, tag_fld//'.'//lab6, vec6)

    open(ids4avs,file=tag_fld//'.fld',form='formatted')
      write(ids4avs,'(a)')    '# AVS field data (by gavs__avs06f)'
      write(ids4avs,'(a)')    '#'
      write(ids4avs,'(a,i5)') 'ndim = ',3
      write(ids4avs,'(a,i5)') 'dim1 = ',n1
      write(ids4avs,'(a,i5)') 'dim2 = ',n2
      write(ids4avs,'(a,i5)') 'dim3 = ',n3
      write(ids4avs,'(a,i5)') 'nspace = ',3
      write(ids4avs,'(a,i5)') 'veclen = ',nvec
      write(ids4avs,'(a)')    'data = float'
      write(ids4avs,'(a)')    'field = irregular'
      write(ids4avs,'(a)')    'label = '//lab1
      write(ids4avs,'(a)')    'label = '//lab2
      write(ids4avs,'(a)')    'label = '//lab3
      write(ids4avs,'(a)')    'label = '//lab4
      write(ids4avs,'(a)')    'label = '//lab5
      write(ids4avs,'(a)')    'label = '//lab6
      write(ids4avs,'(a)')    '#'
      write(ids4avs,'(a)')    'coord 1 file='//tag_coord//'.x filetype=unformatted'
      write(ids4avs,'(a)')    'coord 2 file='//tag_coord//'.y filetype=unformatted'
      write(ids4avs,'(a)')    'coord 3 file='//tag_coord//'.z filetype=unformatted'
      write(ids4avs,'(a)')    '#'
      write(ids4avs,'(a)')    &
                     'variable 1 file='//tag_fld//'.'//lab1//' filetype=unformatted'
      write(ids4avs,'(a)')    &
                     'variable 2 file='//tag_fld//'.'//lab2//' filetype=unformatted'
      write(ids4avs,'(a)')    &
                     'variable 3 file='//tag_fld//'.'//lab3//' filetype=unformatted'
      write(ids4avs,'(a)')    &
                     'variable 4 file='//tag_fld//'.'//lab4//' filetype=unformatted'
      write(ids4avs,'(a)')    &
                     'variable 5 file='//tag_fld//'.'//lab5//' filetype=unformatted'
      write(ids4avs,'(a)')    &
                     'variable 6 file='//tag_fld//'.'//lab6//' filetype=unformatted'
    close(ids4avs)
  end subroutine avs06f
    

  subroutine avs07f(ids4avs,       &
                   tag_coord,      &
                   tag_fld,        &
                   n1, n2, n3,     &
                   vec1, lab1,     &
                   vec2, lab2,     &
                   vec3, lab3,     &
                   vec4, lab4,     &
                   vec5, lab5,     &
                   vec6, lab6,     &
                   vec7, lab7      &
                   )
    integer, intent(in) :: ids4avs
    character(len=*), intent(in) :: tag_coord, tag_fld
    integer, intent(in) :: n1, n2, n3
    real(SP), dimension(n1,n2,n3), intent(in) :: vec1
    real(SP), dimension(n1,n2,n3), intent(in) :: vec2
    real(SP), dimension(n1,n2,n3), intent(in) :: vec3
    real(SP), dimension(n1,n2,n3), intent(in) :: vec4
    real(SP), dimension(n1,n2,n3), intent(in) :: vec5
    real(SP), dimension(n1,n2,n3), intent(in) :: vec6
    real(SP), dimension(n1,n2,n3), intent(in) :: vec7
    character(len=*), intent(in) :: lab1
    character(len=*), intent(in) :: lab2
    character(len=*), intent(in) :: lab3
    character(len=*), intent(in) :: lab4
    character(len=*), intent(in) :: lab5
    character(len=*), intent(in) :: lab6
    character(len=*), intent(in) :: lab7
!------------------------------------------------------------
! SUBROUTINE AVS07F
!------------------------------------------------------------
    integer :: nvec = 7

    call write_3d_data(ids4avs, n1, n2, n3, tag_fld//'.'//lab1, vec1)
    call write_3d_data(ids4avs, n1, n2, n3, tag_fld//'.'//lab2, vec2)
    call write_3d_data(ids4avs, n1, n2, n3, tag_fld//'.'//lab3, vec3)
    call write_3d_data(ids4avs, n1, n2, n3, tag_fld//'.'//lab4, vec4)
    call write_3d_data(ids4avs, n1, n2, n3, tag_fld//'.'//lab5, vec5)
    call write_3d_data(ids4avs, n1, n2, n3, tag_fld//'.'//lab6, vec6)
    call write_3d_data(ids4avs, n1, n2, n3, tag_fld//'.'//lab7, vec7)

    open(ids4avs,file=tag_fld//'.fld',form='formatted')
      write(ids4avs,'(a)')    '# AVS field data (by gavs__avs07f)'
      write(ids4avs,'(a)')    '#'
      write(ids4avs,'(a,i5)') 'ndim = ',3
      write(ids4avs,'(a,i5)') 'dim1 = ',n1
      write(ids4avs,'(a,i5)') 'dim2 = ',n2
      write(ids4avs,'(a,i5)') 'dim3 = ',n3
      write(ids4avs,'(a,i5)') 'nspace = ',3
      write(ids4avs,'(a,i5)') 'veclen = ',nvec
      write(ids4avs,'(a)')    'data = float'
      write(ids4avs,'(a)')    'field = irregular'
      write(ids4avs,'(a)')    'label = '//lab1
      write(ids4avs,'(a)')    'label = '//lab2
      write(ids4avs,'(a)')    'label = '//lab3
      write(ids4avs,'(a)')    'label = '//lab4
      write(ids4avs,'(a)')    'label = '//lab5
      write(ids4avs,'(a)')    'label = '//lab6
      write(ids4avs,'(a)')    'label = '//lab7
      write(ids4avs,'(a)')    '#'
      write(ids4avs,'(a)')    'coord 1 file='//tag_coord//'.x filetype=unformatted'
      write(ids4avs,'(a)')    'coord 2 file='//tag_coord//'.y filetype=unformatted'
      write(ids4avs,'(a)')    'coord 3 file='//tag_coord//'.z filetype=unformatted'
      write(ids4avs,'(a)')    '#'
      write(ids4avs,'(a)')    &
                     'variable 1 file='//tag_fld//'.'//lab1//' filetype=unformatted'
      write(ids4avs,'(a)')    &
                     'variable 2 file='//tag_fld//'.'//lab2//' filetype=unformatted'
      write(ids4avs,'(a)')    &
                     'variable 3 file='//tag_fld//'.'//lab3//' filetype=unformatted'
      write(ids4avs,'(a)')    &
                     'variable 4 file='//tag_fld//'.'//lab4//' filetype=unformatted'
      write(ids4avs,'(a)')    &
                     'variable 5 file='//tag_fld//'.'//lab5//' filetype=unformatted'
      write(ids4avs,'(a)')    &
                     'variable 6 file='//tag_fld//'.'//lab6//' filetype=unformatted'
      write(ids4avs,'(a)')    &
                     'variable 7 file='//tag_fld//'.'//lab7//' filetype=unformatted'
    close(ids4avs)
  end subroutine avs07f
    

  subroutine avs08f(ids4avs,       &
                   tag_coord,      &
                   tag_fld,        &
                   n1, n2, n3,     &
                   vec1, lab1,     &
                   vec2, lab2,     &
                   vec3, lab3,     &
                   vec4, lab4,     &
                   vec5, lab5,     &
                   vec6, lab6,     &
                   vec7, lab7,     &
                   vec8, lab8      &
                   )
    integer, intent(in) :: ids4avs
    character(len=*), intent(in) :: tag_coord, tag_fld
    integer, intent(in) :: n1, n2, n3
    real(SP), dimension(n1,n2,n3), intent(in) :: vec1
    real(SP), dimension(n1,n2,n3), intent(in) :: vec2
    real(SP), dimension(n1,n2,n3), intent(in) :: vec3
    real(SP), dimension(n1,n2,n3), intent(in) :: vec4
    real(SP), dimension(n1,n2,n3), intent(in) :: vec5
    real(SP), dimension(n1,n2,n3), intent(in) :: vec6
    real(SP), dimension(n1,n2,n3), intent(in) :: vec7
    real(SP), dimension(n1,n2,n3), intent(in) :: vec8
    character(len=*), intent(in) :: lab1
    character(len=*), intent(in) :: lab2
    character(len=*), intent(in) :: lab3
    character(len=*), intent(in) :: lab4
    character(len=*), intent(in) :: lab5
    character(len=*), intent(in) :: lab6
    character(len=*), intent(in) :: lab7
    character(len=*), intent(in) :: lab8
!------------------------------------------------------------
! SUBROUTINE AVS08F
!------------------------------------------------------------
    integer :: nvec = 8

    call write_3d_data(ids4avs, n1, n2, n3, tag_fld//'.'//lab1, vec1)
    call write_3d_data(ids4avs, n1, n2, n3, tag_fld//'.'//lab2, vec2)
    call write_3d_data(ids4avs, n1, n2, n3, tag_fld//'.'//lab3, vec3)
    call write_3d_data(ids4avs, n1, n2, n3, tag_fld//'.'//lab4, vec4)
    call write_3d_data(ids4avs, n1, n2, n3, tag_fld//'.'//lab5, vec5)
    call write_3d_data(ids4avs, n1, n2, n3, tag_fld//'.'//lab6, vec6)
    call write_3d_data(ids4avs, n1, n2, n3, tag_fld//'.'//lab7, vec7)
    call write_3d_data(ids4avs, n1, n2, n3, tag_fld//'.'//lab8, vec8)

    open(ids4avs,file=tag_fld//'.fld',form='formatted')
      write(ids4avs,'(a)')    '# AVS field data (by gavs__avs08f)'
      write(ids4avs,'(a)')    '#'
      write(ids4avs,'(a,i5)') 'ndim = ',3
      write(ids4avs,'(a,i5)') 'dim1 = ',n1
      write(ids4avs,'(a,i5)') 'dim2 = ',n2
      write(ids4avs,'(a,i5)') 'dim3 = ',n3
      write(ids4avs,'(a,i5)') 'nspace = ',3
      write(ids4avs,'(a,i5)') 'veclen = ',nvec
      write(ids4avs,'(a)')    'data = float'
      write(ids4avs,'(a)')    'field = irregular'
      write(ids4avs,'(a)')    'label = '//lab1
      write(ids4avs,'(a)')    'label = '//lab2
      write(ids4avs,'(a)')    'label = '//lab3
      write(ids4avs,'(a)')    'label = '//lab4
      write(ids4avs,'(a)')    'label = '//lab5
      write(ids4avs,'(a)')    'label = '//lab6
      write(ids4avs,'(a)')    'label = '//lab7
      write(ids4avs,'(a)')    'label = '//lab8
      write(ids4avs,'(a)')    '#'
      write(ids4avs,'(a)')    'coord 1 file='//tag_coord//'.x filetype=unformatted'
      write(ids4avs,'(a)')    'coord 2 file='//tag_coord//'.y filetype=unformatted'
      write(ids4avs,'(a)')    'coord 3 file='//tag_coord//'.z filetype=unformatted'
      write(ids4avs,'(a)')    '#'
      write(ids4avs,'(a)')    &
                     'variable 1 file='//tag_fld//'.'//lab1//' filetype=unformatted'
      write(ids4avs,'(a)')    &
                     'variable 2 file='//tag_fld//'.'//lab2//' filetype=unformatted'
      write(ids4avs,'(a)')    &
                     'variable 3 file='//tag_fld//'.'//lab3//' filetype=unformatted'
      write(ids4avs,'(a)')    &
                     'variable 4 file='//tag_fld//'.'//lab4//' filetype=unformatted'
      write(ids4avs,'(a)')    &
                     'variable 5 file='//tag_fld//'.'//lab5//' filetype=unformatted'
      write(ids4avs,'(a)')    &
                     'variable 6 file='//tag_fld//'.'//lab6//' filetype=unformatted'
      write(ids4avs,'(a)')    &
                     'variable 7 file='//tag_fld//'.'//lab7//' filetype=unformatted'
      write(ids4avs,'(a)')    &
                     'variable 8 file='//tag_fld//'.'//lab8//' filetype=unformatted'
    close(ids4avs)
  end subroutine avs08f
    

  subroutine avs09f(ids4avs,          &
                   tag_coord,         &
                   tag_fld,           &
                   n1, n2, n3,        &
                   vec1, lab1,        &
                   vec2, lab2,        &
                   vec3, lab3,        &
                   vec4, lab4,        &
                   vec5, lab5,        &
                   vec6, lab6,        &
                   vec7, lab7,        &
                   vec8, lab8,        &
                   vec9, lab9         &
                   )
    integer, intent(in) :: ids4avs
    character(len=*), intent(in) :: tag_coord, tag_fld
    integer, intent(in) :: n1, n2, n3
    real(SP), dimension(n1,n2,n3), intent(in) :: vec1
    real(SP), dimension(n1,n2,n3), intent(in) :: vec2
    real(SP), dimension(n1,n2,n3), intent(in) :: vec3
    real(SP), dimension(n1,n2,n3), intent(in) :: vec4
    real(SP), dimension(n1,n2,n3), intent(in) :: vec5
    real(SP), dimension(n1,n2,n3), intent(in) :: vec6
    real(SP), dimension(n1,n2,n3), intent(in) :: vec7
    real(SP), dimension(n1,n2,n3), intent(in) :: vec8
    real(SP), dimension(n1,n2,n3), intent(in) :: vec9
    character(len=*), intent(in) :: lab1
    character(len=*), intent(in) :: lab2
    character(len=*), intent(in) :: lab3
    character(len=*), intent(in) :: lab4
    character(len=*), intent(in) :: lab5
    character(len=*), intent(in) :: lab6
    character(len=*), intent(in) :: lab7
    character(len=*), intent(in) :: lab8
    character(len=*), intent(in) :: lab9
!------------------------------------------------------------
! SUBROUTINE AVS09F
!------------------------------------------------------------
    integer :: nvec = 9

    call write_3d_data(ids4avs, n1, n2, n3, tag_fld//'.'//lab1, vec1)
    call write_3d_data(ids4avs, n1, n2, n3, tag_fld//'.'//lab2, vec2)
    call write_3d_data(ids4avs, n1, n2, n3, tag_fld//'.'//lab3, vec3)
    call write_3d_data(ids4avs, n1, n2, n3, tag_fld//'.'//lab4, vec4)
    call write_3d_data(ids4avs, n1, n2, n3, tag_fld//'.'//lab5, vec5)
    call write_3d_data(ids4avs, n1, n2, n3, tag_fld//'.'//lab6, vec6)
    call write_3d_data(ids4avs, n1, n2, n3, tag_fld//'.'//lab7, vec7)
    call write_3d_data(ids4avs, n1, n2, n3, tag_fld//'.'//lab8, vec8)
    call write_3d_data(ids4avs, n1, n2, n3, tag_fld//'.'//lab9, vec9)

    open(ids4avs,file=tag_fld//'.fld',form='formatted')
      write(ids4avs,'(a)')    '# AVS field data (by gavs__avs09f)'
      write(ids4avs,'(a)')    '#'
      write(ids4avs,'(a,i5)') 'ndim = ',3
      write(ids4avs,'(a,i5)') 'dim1 = ',n1
      write(ids4avs,'(a,i5)') 'dim2 = ',n2
      write(ids4avs,'(a,i5)') 'dim3 = ',n3
      write(ids4avs,'(a,i5)') 'nspace = ',3
      write(ids4avs,'(a,i5)') 'veclen = ',nvec
      write(ids4avs,'(a)')    'data = float'
      write(ids4avs,'(a)')    'field = irregular'
      write(ids4avs,'(a)')    'label = '//lab1
      write(ids4avs,'(a)')    'label = '//lab2
      write(ids4avs,'(a)')    'label = '//lab3
      write(ids4avs,'(a)')    'label = '//lab4
      write(ids4avs,'(a)')    'label = '//lab5
      write(ids4avs,'(a)')    'label = '//lab6
      write(ids4avs,'(a)')    'label = '//lab7
      write(ids4avs,'(a)')    'label = '//lab8
      write(ids4avs,'(a)')    'label = '//lab9
      write(ids4avs,'(a)')    '#'
      write(ids4avs,'(a)')    'coord 1 file='//tag_coord//'.x filetype=unformatted'
      write(ids4avs,'(a)')    'coord 2 file='//tag_coord//'.y filetype=unformatted'
      write(ids4avs,'(a)')    'coord 3 file='//tag_coord//'.z filetype=unformatted'
      write(ids4avs,'(a)')    '#'
      write(ids4avs,'(a)')    &
                     'variable 1 file='//tag_fld//'.'//lab1//' filetype=unformatted'
      write(ids4avs,'(a)')    &
                     'variable 2 file='//tag_fld//'.'//lab2//' filetype=unformatted'
      write(ids4avs,'(a)')    &
                     'variable 3 file='//tag_fld//'.'//lab3//' filetype=unformatted'
      write(ids4avs,'(a)')    &
                     'variable 4 file='//tag_fld//'.'//lab4//' filetype=unformatted'
      write(ids4avs,'(a)')    &
                     'variable 5 file='//tag_fld//'.'//lab5//' filetype=unformatted'
      write(ids4avs,'(a)')    &
                     'variable 6 file='//tag_fld//'.'//lab6//' filetype=unformatted'
      write(ids4avs,'(a)')    &
                     'variable 7 file='//tag_fld//'.'//lab7//' filetype=unformatted'
      write(ids4avs,'(a)')    &
                     'variable 8 file='//tag_fld//'.'//lab8//' filetype=unformatted'
      write(ids4avs,'(a)')    &
                     'variable 9 file='//tag_fld//'.'//lab9//' filetype=unformatted'
    close(ids4avs)
  end subroutine avs09f
    

  subroutine avs10f(ids4avs,         &
                    tag_coord,       &
                    tag_fld,         &
                    n1, n2, n3,      &
                    vec01, lab01,    &
                    vec02, lab02,    &
                    vec03, lab03,    &
                    vec04, lab04,    &
                    vec05, lab05,    &
                    vec06, lab06,    &
                    vec07, lab07,    &
                    vec08, lab08,    &
                    vec09, lab09,    &
                    vec10, lab10     &
                   )
    integer, intent(in) :: ids4avs
    character(len=*), intent(in) :: tag_coord, tag_fld
    integer, intent(in) :: n1, n2, n3
    real(SP), dimension(n1,n2,n3), intent(in) :: vec01
    real(SP), dimension(n1,n2,n3), intent(in) :: vec02
    real(SP), dimension(n1,n2,n3), intent(in) :: vec03
    real(SP), dimension(n1,n2,n3), intent(in) :: vec04
    real(SP), dimension(n1,n2,n3), intent(in) :: vec05
    real(SP), dimension(n1,n2,n3), intent(in) :: vec06
    real(SP), dimension(n1,n2,n3), intent(in) :: vec07
    real(SP), dimension(n1,n2,n3), intent(in) :: vec08
    real(SP), dimension(n1,n2,n3), intent(in) :: vec09
    real(SP), dimension(n1,n2,n3), intent(in) :: vec10
    character(len=*), intent(in) :: lab01
    character(len=*), intent(in) :: lab02
    character(len=*), intent(in) :: lab03
    character(len=*), intent(in) :: lab04
    character(len=*), intent(in) :: lab05
    character(len=*), intent(in) :: lab06
    character(len=*), intent(in) :: lab07
    character(len=*), intent(in) :: lab08
    character(len=*), intent(in) :: lab09
    character(len=*), intent(in) :: lab10
!------------------------------------------------------------
! SUBROUTINE AVS10F
!------------------------------------------------------------
    integer :: nvec = 10

    call write_3d_data(ids4avs,n1,n2,n3,tag_fld//'.'//lab01,vec01)
    call write_3d_data(ids4avs,n1,n2,n3,tag_fld//'.'//lab02,vec02)
    call write_3d_data(ids4avs,n1,n2,n3,tag_fld//'.'//lab03,vec03)
    call write_3d_data(ids4avs,n1,n2,n3,tag_fld//'.'//lab04,vec04)
    call write_3d_data(ids4avs,n1,n2,n3,tag_fld//'.'//lab05,vec05)
    call write_3d_data(ids4avs,n1,n2,n3,tag_fld//'.'//lab06,vec06)
    call write_3d_data(ids4avs,n1,n2,n3,tag_fld//'.'//lab07,vec07)
    call write_3d_data(ids4avs,n1,n2,n3,tag_fld//'.'//lab08,vec08)
    call write_3d_data(ids4avs,n1,n2,n3,tag_fld//'.'//lab09,vec09)
    call write_3d_data(ids4avs,n1,n2,n3,tag_fld//'.'//lab10,vec10)

    open(ids4avs,file=tag_fld//'.fld',form='formatted')
      write(ids4avs,'(a)')    '# AVS field data (by gavs__avs10f)'
      write(ids4avs,'(a)')    '#'
      write(ids4avs,'(a,i5)') 'ndim = ',3
      write(ids4avs,'(a,i5)') 'dim1 = ',n1
      write(ids4avs,'(a,i5)') 'dim2 = ',n2
      write(ids4avs,'(a,i5)') 'dim3 = ',n3
      write(ids4avs,'(a,i5)') 'nspace = ',3
      write(ids4avs,'(a,i5)') 'veclen = ',nvec
      write(ids4avs,'(a)')    'data = float'
      write(ids4avs,'(a)')    'field = irregular'
      write(ids4avs,'(a)')    'label = '//lab01
      write(ids4avs,'(a)')    'label = '//lab02
      write(ids4avs,'(a)')    'label = '//lab03
      write(ids4avs,'(a)')    'label = '//lab04
      write(ids4avs,'(a)')    'label = '//lab05
      write(ids4avs,'(a)')    'label = '//lab06
      write(ids4avs,'(a)')    'label = '//lab07
      write(ids4avs,'(a)')    'label = '//lab08
      write(ids4avs,'(a)')    'label = '//lab09
      write(ids4avs,'(a)')    'label = '//lab10
      write(ids4avs,'(a)')    '#'
      write(ids4avs,'(a)')    'coord 1 file='//tag_coord//'.x filetype=unformatted'
      write(ids4avs,'(a)')    'coord 2 file='//tag_coord//'.y filetype=unformatted'
      write(ids4avs,'(a)')    'coord 3 file='//tag_coord//'.z filetype=unformatted'
      write(ids4avs,'(a)')    '#'
      write(ids4avs,'(a)')    &
                     'variable 1 file='//tag_fld//'.'//lab01//' filetype=unformatted'
      write(ids4avs,'(a)')    &
                     'variable 2 file='//tag_fld//'.'//lab02//' filetype=unformatted'
      write(ids4avs,'(a)')    &
                     'variable 3 file='//tag_fld//'.'//lab03//' filetype=unformatted'
      write(ids4avs,'(a)')    &
                     'variable 4 file='//tag_fld//'.'//lab04//' filetype=unformatted'
      write(ids4avs,'(a)')    &
                     'variable 5 file='//tag_fld//'.'//lab05//' filetype=unformatted'
      write(ids4avs,'(a)')    &
                     'variable 6 file='//tag_fld//'.'//lab06//' filetype=unformatted'
      write(ids4avs,'(a)')    &
                     'variable 7 file='//tag_fld//'.'//lab07//' filetype=unformatted'
      write(ids4avs,'(a)')    &
                     'variable 8 file='//tag_fld//'.'//lab08//' filetype=unformatted'
      write(ids4avs,'(a)')    &
                     'variable 9 file='//tag_fld//'.'//lab09//' filetype=unformatted'
      write(ids4avs,'(a)')    &
                     'variable 10 file='//tag_fld//'.'//lab10//' filetype=unformatted'
    close(ids4avs)
  end subroutine avs10f
    

  subroutine write_3d_data(ids4avs,n1,n2,n3,file_name,data)
    integer, intent(in) :: ids4avs
    integer, intent(in) :: n1, n2, n3
    character(len=*), intent(in) :: file_name
    real(SP), dimension(n1,n2,n3), intent(in) :: data
!------------------------------------------------------------
! SUBROUTINE WRITE_3D_DATA
!------------------------------------------------------------
    open(ids4avs,file=file_name,form='unformatted')
       write(ids4avs) data
    close(ids4avs)
  end subroutine write_3d_data


!==============
!    Public    
!==============

    
  subroutine gavs__coord(ids4avs, tag_coord,      &
                         n1, n2, n3,              &
                         x, y, z)
    integer, intent(in) :: ids4avs
    character(len=*), intent(in) :: tag_coord
    integer, intent(in) :: n1, n2, n3
    real(SP), dimension(n1,n2,n3), intent(in) :: x, y, z
!============================================================
! SUBROUTINE GAVS__COORD
!============================================================
    call write_3d_data(ids4avs, n1, n2, n3, tag_coord//'.x', x)
    call write_3d_data(ids4avs, n1, n2, n3, tag_coord//'.y', y)
    call write_3d_data(ids4avs, n1, n2, n3, tag_coord//'.z', z)
  end subroutine gavs__coord

end module gavs
