!-----------------------------------------------------------------------------
! KINDANB: kinematic dynamo in a box
!-----------------------------------------------------------------------------
! main.f90
!              2007.08.12: Minor revision, for Les Houces Summer School.
!              2007.07.22: Minor revision, for Les Houces Summer School.
!              2006.09.25: Revised the module structure.
!              2005.06.01: Created by Akira Kageyama.
!-----------------------------------------------------------------------------

program main
!=============================================================================
! PROGAM MAIN                      A Kinematic Dynamo Code in the Box Geometry
!=============================================================================
  !                                        Akira Kageyama (kage@jamstec.go.jp)
  !---------------------------------------------------------------------------
  use constants             !                              numerical constants
  use ut                    !                                utility functions
  use grid                  !                                        grid mesh
  use namelist              !                                   namelis loader
  use field                 !                   field operators and operations
  use solenoidal            !               makes divergence-free vector field
! use gavs                  !              generator of avs-field data (*.fld)
! use avsdata               !              make and save 3D data in avs format
  use slicedata             !                         generate 2-d sliced data
  use rk4                   !         4th order runge-kutta integration method
  implicit none             !-------------------------------------------------

  real(DP), parameter :: ABNORMALLY_LARGE   = 1.e10_DP
  real(DP), parameter :: SUFFICIENTLY_SMALL = 1.e-12_DP

  integer  :: nloop, karte
  real(DP) :: eta, mag_energy, dt, time
  real(DP) :: div_vel_max, div_mag_max

  type(field__vector3d_) :: vel
  type(field__vector3d_) :: mag

  call grid__initialize
  call namelist__read
! call avsdata__initialize
  call slicedata__initialize

  vel = 0.0_DP
  mag = 0.0_DP
  time = 0.0_DP
  nloop = 0

  eta = 1.0_DP / namelist__double('Magnetic_reynolds')
  
  vel = solenoidal__abc_flow()

  mag = solenoidal__sample01(l=4,m=3,n=2)
  mag = mag + solenoidal__sample01(l=2,m=2,n=1)
  mag = mag + solenoidal__sample01(l=1,m=1,n=1)

  div_vel_max = maxval(.div.vel)
  div_mag_max = maxval(.div.mag)

  call ut__message('initial check: div(V) = ', div_vel_max)
  call ut__message('initial check: div(B) = ', div_mag_max)

  if (div_vel_max>SUFFICIENTLY_SMALL) call ut__fatal('div(V) is too large.')
  if (div_mag_max>SUFFICIENTLY_SMALL) call ut__fatal('div(B) is too large.')

  call ut__message('#vel_energy: ', time, .energyintegral.vel) 
  call ut__message('#mag_energy: ', time, .energyintegral.mag) 

  dt = iCFL_condition()

  if (nloop < namelist__integer('Total_nloop')) karte = KARTE_FINE

  do while(karte==KARTE_FINE)
     call rk4__advance(time,dt,eta,vel,mag)
     nloop = nloop + 1
     if (nloop >= namelist__integer('Total_nloop')) karte = KARTE_LOOP_MAX
     if (mod(nloop,100)==0) then
        mag_energy = .energyintegral.mag
        if ( mag_energy > ABNORMALLY_LARGE ) karte = KARTE_OVERFLOW
        call ut__message('#mag_energy: ', nloop, time, mag_energy) 
     end if
!    call avsdata__write(vel,mag,nloop,time)
     call slicedata__write(vel,mag,nloop,time)
  end do

  call iFinish

contains

  !- - - - - - - - - - - - - - - - - - - - - - - - - - - - 
  function iCFL_condition()
    real(DP) :: iCFL_condition
  !- - - - - - - - - - - - - - - - - - - - - - - - - - - - 
    real(DP), parameter :: ALMOST_ZERO = 1.e-20_DP
    real(DP) :: grid_size_minimum, vmax, dt_vel, dt_eta, dt

    grid_size_minimum = min(grid__delta%x,grid__delta%y,grid__delta%z)

    vmax = maxval(sqrt(vel%x**2+vel%y**2+vel%z**2))
    vmax = max(vmax,ALMOST_ZERO)

    dt_vel = 0.8_DP*grid_size_minimum/vmax
    dt_eta = 0.2_DP*(grid_size_minimum**2)/eta

    dt = min(dt_vel, dt_eta)

    call ut__message('               vmax = ', vmax   )
    call ut__message('                eta = ', eta    )
    call ut__message('             dt_vel = ', dt_vel )
    call ut__message('             dt_eta = ', dt_eta )
    call ut__message('          -->    dt = ', dt     )

    iCFL_condition = dt

  end function iCFL_condition


  !- - - - - - - - - - - - - - - - - - - - - - - - - - - - 
  subroutine iFinish
  !- - - - - - - - - - - - - - - - - - - - - - - - - - - - 
    select case (karte)
    case (KARTE_FINE)
       call ut__message('#',"Successfully finished.")
    case (KARTE_LOOP_MAX)
       call ut__message('=',"Have reached the max nloop: ", nloop)
    case (KARTE_TIME_OUT)
       call ut__message('-',"Time out at nloop = ", nloop)
    case (KARTE_OVERFLOW)
       call ut__message('%',"Overflow at nloop = ", nloop)
    case (KARTE_UNDERFLOW)
       call ut__message('%',"Underflow at nloop = ", nloop)
    case default
       call ut__message('?',"Stopped at nloop = ", nloop)
    end select
  end subroutine iFinish

end program main
